# comic-get.pl - Utility for fetching comics from the internet
# Copyright (C) 2004-2007 Tommi Saviranta <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package ComicGet::Fetch;

use strict;
use warnings FATAL => 'all';

use IO::Socket;

use ComicGet::Config;
use ComicGet::Web;


my $file_no = 0;
my $act_line;


=item list_to_hash
@: List
r: Hash
=cut
sub list_to_hash(@) {
	my %hash;
	foreach my $line (@_) {
		my (@t) = split(/\t/, $line);
		$hash{$t[0]} = $t[1];
	}
	return %hash;
}


=item get
0: Page
1: Type
@: Params
r: Target
=cut
sub get($$@) {
	my $page = shift;
	my $type = shift;
	my $mparam = shift;
	my ($uri, $t0, $t1);
	my ($otag, $stag, $etag, $opt);
	$mparam =~ s/ /.*?/;

	if ($type eq "image") {
		$otag = "<img";
		$etag = ">";
		$opt = "src";
	}
	elsif ($type eq "link") {
		$otag = "<a";
		$etag = "</a>";
		$opt = "href";
	} else {
		die("Unknown type for get: $type");
	}

	# Assume no '<'s in URIs
	($_) = $page =~ /.*$otag(\s.*?$mparam.*?)$etag/is;
	return undef if (! defined($_));

	# look for all rules
	foreach my $rule (@_) {
		return undef if (! (/$rule/is));
	}
	
	($_) = /\s$opt\s*=\s*(.*)/is;
	s/\n//g;
	($t0, $t1) = /(.)(.*)/;
	# Was it 'foo="uri"', 'foo="uri"' or "foo=uri"?
	if ($t0 =~ /['"]/) {
		($uri) = $t1 =~ /(.*?)$t0/;
	} else {
		$t1 = $t0 . $t1;
		($uri) = $t1 =~ /(.*?)\s/;
	}
	
	return &ComicGet::Web::complete_uri($uri);
}


=item follow_rules
Follow rules to fetch target.

0: Rules
1: Target file
r: 1 on success.
=cut
sub follow_rules($) {
	my ($rules, $target) = @_;
	my ($data, $uri_id);
	my $title;
	my @uri_list;

	$file_no++;

	foreach my $line (split(/\n/, $rules)) {
		# Remove comments and trailing whitespaces
		$line =~ s/\\#/\n/;	# Temporarily convert escaped #s into lf
		$line =~ s/#.*//s;	# Strip comments
		$line =~ s/\s+$//s;	# Strip whitespaces
		$line =~ s/\n/#/s;	# Put #s back

		next if (length($line) == 0);
	
		my (@stuff) = split(/\s+/, $line);
		my $cmd = shift @stuff;
		my $opt = join(" ", @stuff);

		if ($cmd eq "title") {
			&ComicGet::Web::clear_referer();
			&ComicGet::Web::clear_cookies();
			$title = $opt;
			undef @uri_list;
		}
		
		elsif ($cmd eq "uri") {
			my ($host) = $stuff[0] =~ /^(.*?\/\/.*?\/)/;
			$act_line = "Get: $file_no $host $title";
			print "$act_line\r";
			flush STDOUT;

			push(@uri_list, $stuff[0]);
		}
		
		elsif ($cmd eq "fetch") {
			print "$act_line [fetch]\r";
			flush STDOUT;
			undef $uri_id;
			my $uri;
			if ($#stuff == -1) {
				($#uri_list == 0) || die("Too many URIs!");
				$uri = $uri_list[0];
			} else {
				foreach my $id (@stuff) {
					foreach my $cand (@uri_list) {
						if ($cand =~ /^$id\t/) {
							my @t = split(/\t/,
								$cand, 2);
							$uri = $t[1];
							$uri_id = $id;
							last;
						}
					}
				}
			}
			if (! defined($uri)) {
				print "\nW: No appropriate URI found. " .
					"Skipping...\n";
				return 0;
			}
			&ComicGet::Web::set_actline($act_line);
			$data = &ComicGet::Web::http_get($uri, 80);
			&ComicGet::Web::set_actline(undef);
			if ($ComicGet::Web::ret_code != $ComicGet::Web::OK) {
				print "\n";
				&ComicGet::Web::print_error();
				print "W: Skipping...\n";
				return 0;
			}
			my $size = &ComicGet::Common::human_readable(
					length($data));
			print "$act_line [$size]  \n";
			undef @uri_list;
		}
		
		elsif ($cmd eq "image") {
			my $type = "";
			$type = shift @stuff if ($#stuff > 0);
			my $t = &get($data, "image", @stuff);
			next if (! defined($t));
			push(@uri_list, "$type\t$t");
		}

		elsif ($cmd eq "link") {
			my $t = &get($data, "link", @stuff);
			next if (! defined($t));
			push(@uri_list, $t);
		}
		
		elsif ($cmd eq "save") {
			if (! defined($data)) {
				die("Data undefined!");
			}

			&ComicGet::Config::read();
			my $path = $ComicGet::Config::target_dir;

			my $orig_file = shift @stuff;
			if (! defined($target)) {
				print "\nW: Database has target filename " .
					"missing. Using $orig_file instead.\n";
				$target = $orig_file;
			}
			my $file = "$path/$target";

			my $size = length($data);
			my $convert;
			foreach my $extra (@stuff) {
				if ($extra =~ /^convert:/) {
					my ($t) = $extra =~ /convert:(.*)/;
					foreach my $id (split(/,/, $t)) {
						if ($id eq $uri_id) {
							$convert = $id;
							last;
						}
					}
				}
			}

			$size = &ComicGet::Common::human_readable($size);

			if (defined($convert)) {
				print "$act_line [convert]  \r";
				flush STDOUT;
				open(PIPE, "| convert - $file")
						or die("Cannot convert: $!");
				print PIPE $data;
				close(PIPE);
			} else {
				&ComicGet::Common::save_file("$file", $data)
					or die("Cannot write \"$file\": " .
						&ComicGet::Common::error);
			}

			my ($trim) = $file =~ /.*\/(.*)$/;
			print "$act_line [$size] $trim  \n";
			flush STDOUT;
		}
		
		elsif ($cmd eq "stop") {
			die("Stopped");
		}

		elsif ($cmd eq "--info--") {
			return 1;
		}
		
		else {
			die("Unrecognized command: $cmd");
		}
	}

	print "\nW: Rulefile ended unexpectedly.\n";
	return 0;
}


=item get_file
Get filename from rules file.

0: Rule info
r: Filename
=cut
sub get_file($) {
	my $rules = shift;

	foreach my $line (split(/\n/, $rules)) {
		my @t = split(/\s+/, $line);
		(defined($t[0]) && $t[0] eq "save") && return $t[1];
	}
	return undef;
}


1;
